<?php
namespace Geral\Util;

/**
 * Handler de Mensagens.
 *
 * Classe responsavel por fazer o tratamento das mensagens vindas do logic. Esta classe deve ser capaz de identificar
 * os diferentes tipos de mensagens (mensagens de validacao, mensgens de excecoes da camada logica, excecoes vindas de
 * erro dos componentes da aplicacao, como banco de dados, etc).
 * Inicialmente, ela identifica tres grupos diferentes de mensagens: primeiro: Excecoes que sao instancias de
 * \Geral\Logic\Exception, que serao consideradas validacoes de regras de negocios; segundo: Excecoes que carregam o
 * identificador do Acumulador de mensagens, que significa que o codigo produziu um conjunto de mensagens, e que estas
 * deverao ser exibidas. Para este caso, as proprias mensagens deverao ser capazes de apresentar a esta classe qual tipo
 * de mensagem deve ser exibida; terceiro, mensagens comuns, que serao consideradas excecoes, como erros de banco de
 * dados, falha ao abrir arquivos de texto, etc. Estes serao sempre tratados como mensagens de erro.
 * A classe ainda conta com um metodo que deve tratar todas as mensagens do acumulador, sem que este necessite ser
 * enviado como excecao.
 *
 * Created by JetBrains PhpStorm.
 * User: paulopereira
 * Date: 29/11/12
 * Time: 13:14
 * To change this template use File | Settings | File Templates.
 */
class TratamentoDeMensagens
{

    private static $abort = false;

    /**
     * Constante usada para definir se a classe deve ou nao exibir mensagens de erro.
     *
     * @var bool
     */
    private static $EXIBIR_MENSAGENS_DE_ERRO = true;

    /**
     * Exibe as mensagens capturadas pelas excecoes.
     *
     * @static
     * @param \Exception $e
     * @param bool $abort
     */
    public static function exibirMensagens(\Exception $e, $abort = false)
    {
        self::$abort = $abort;

        if ($e instanceof \Geral\Logic\Exception) {
            self::tratamentoDeExcecaoLogic($e);
        } else if ($e->getMessage() === \Geral\Util\AcumuladorDeMensagens::ID_ACUMULADOR_DE_MENSAGENS) {
            self::tramentoAcumuladorDeMensagens();
        } else {
            self::tratamentoDeExcecao($e);
        }
    }

    /**
     * Metodo utilizado para apresentar as mensagens do Acumulador de mensagens, que nao sejam necessariamente excecoes
     * (na pratica, utilizadas para exibir um conjunto de mensagens que nao se espera que parem a execucao do codigo).
     *
     * @static
     * @param bool $abort
     */
    public static function exibeMensagensAcumuladas($abort = false)
    {
        if ($abort) {
            self::tramentoAcumuladorDeMensagensComAbort();
        } else {
            self::tramentoAcumuladorDeMensagens();
        }
    }

    /**
     * Trata as excecoes lancadas pela aplicacao (excecoes da camada logica da aplicacao).
     *
     * @static
     * @param \Geral\Logic\Exception $e
     */
    private static function tratamentoDeExcecaoLogic(\Geral\Logic\Exception $e)
    {
        try {
            $tkMessage = $e->getMessageName();
            $params    = $e->getParams();
            $mensagem  = self::tkGetMessageByName($tkMessage, $params);
            self::exibirMensagem($mensagem, \Geral\Util\Mensagem::ALERTA);
        } catch (\Exception $e) {
            self::exibirMensagem($e->getMessage());
        }
    }

    /**
     * Trata as excecoes comuns, como erro de banco, arquivos nao encontrados, etc. Somente sera exibida se o parametro
     * $EXIBIR_MENSAGENS_DE_ERRO = true.
     *
     * @static
     * @param \Exception $e
     */
    private static function tratamentoDeExcecao(\Exception $e)
    {
        if (self::$EXIBIR_MENSAGENS_DE_ERRO) {
            self::exibirMensagem($e->getMessage());
        }
    }

    /**
     * Trata o acumulador de mensagens: exibe todas as mensagens acumuladas ate agora.
     *
     * @static
     */
    private static function tramentoAcumuladorDeMensagens()
    {
        while (\Geral\Util\AcumuladorDeMensagens::getInstance()->existemMensagens()) {
            $objMensagem = \Geral\Util\AcumuladorDeMensagens::getInstance()->getMensagem();
            $mensagem    = self::tkGetMessageByName($objMensagem->getTexto(), $objMensagem->getParametros());
            self::exibirMensagem($mensagem, $objMensagem->getTipoMensagem());
        }
    }

    private static function tramentoAcumuladorDeMensagensComAbort()
    {
        $acumulaMensagem = "Erros encontrados: <br>";
        while (AcumuladorDeMensagens::getInstance()->existemMensagens()) {
            /* @var $mensagem Mensagem */
            $mensagem = AcumuladorDeMensagens::getInstance()->getMensagem();
            $parametros = is_array($mensagem->getParametros()) ? $mensagem->getParametros() : array();
            $acumulaMensagem .= '- '. tk_get_message_by_name($mensagem->getTexto(), $parametros)."<br>";
        }

        if ($acumulaMensagem != "Erros encontrados: <br>") {
            tk_abort($acumulaMensagem);
        }
    }

    /**
     * Encapsula as chamadas ao tk_message. Define se a chamada sera feita para mensagens de erro, alerta ou info.
     *
     * @static
     * @param $mensagem
     * @param string $tipo
     */
    private static function exibirMensagem($mensagem, $tipo = \Geral\Util\Mensagem::ERRO)
    {
        if (self::$abort) {
            tk_abort($mensagem);
        } else if ($tipo === \Geral\Util\Mensagem::ERRO) {
            tk_message($mensagem, "error");
        } else if ($tipo === \Geral\Util\Mensagem::ALERTA) {
            tk_message($mensagem, "warning");
        } else {
            tk_message($mensagem);
        }
    }

    /**
     * Encapsula as chamadas ao self::tkGetMessageByName.
     *
     * @static
     * @param $name
     * @param $params
     * @return mixed|null|string
     */
    private static function tkGetMessageByName($name, $params)
    {
        return tk_get_message_by_name($name, $params);
    }

    /**
     * Função criada para exibir mensagens (que não são erros) agrupadas.
     * Foi criada devido ao comportamento da nova versão do SetForms (2.50),
     * que só permite a exibição de uma mensagem na tela.
     * Devido à necessidade de exibição de mensagens do servidor (que não estão
     * cadastradas no metadados), a função exibe o texto passado no 'addMensagem',
     * e não realiza um 'tk_get_message_by_name' como a outra função da classe.
     * Portanto, se for utilizá-la, utilize o tk_get_message_by_name antes de
     * addMensagem.
     *
     * @return null
     */
    public static function exibeMensagensAcumuladasAgrupadas()
    {
        $acumulaMensagem = '';
        while (AcumuladorDeMensagens::getInstance()->existemMensagens()) {
            /* @var $mensagem Mensagem */
            $mensagem = AcumuladorDeMensagens::getInstance()->getMensagem();
            $acumulaMensagem .= '- '. $mensagem->getTexto()."<br>";
        }

        if ($acumulaMensagem != '') {
            tk_message($acumulaMensagem);
        }
    }

}
